//==================================================================
//
//	File name	:	menu.js
//
//	Summary		:	Menu classes
//
//	Ceated		:	2018/12/04
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================
"use strict";

(function (menu) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	menu.ITEM_STYLE_DEFAULT = 0;
	menu.ITEM_STYLE_IMAGE = 1;
	menu.ITEM_STYLE_ICON = 2;
	menu.ITEM_STYLE_CHECKBOX = 3;
	menu.ITEM_STYLE_CASCADE = 4;
	menu.ITEM_STYLE_DELIMITER = 5;
	
	menu.BAR_ITEM_STATUS_DEFAULT = 0;
	menu.BAR_ITEM_STATUS_ACTIVE = 1;
	
	// Grobal variables
	menu.menus = {};
	menu.visibleMenus = new Array();
	
	//==========================================================
	//
	//	Class		:	MenuItem
	//
	//	Summary		:	Class of menu item
	//
	//==========================================================
	menu.MenuItem = class
	{
		//==================================================
		// constructor
		//==================================================
		constructor(iStyle,			// * Item style[int]
					bEnabled,		// Enabled or not[boolean]
					sLabel,			// Label text[String]
					sContents,		// Icon URL or Icon css class[String]
					sGroup,			// Group name(for Radio button)[String]
					bChecked,		// Checked or not[boolean]
					fpClick)		// Function on click
		{
			// Check parameters
			if(typeof iStyle != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, ""),
											"menu.js", "jaswin.menu.MenuItem.constructor");
			
			if((iStyle != menu.ITEM_STYLE_DEFAULT)
			&& (iStyle != menu.ITEM_STYLE_IMAGE)
			&& (iStyle != menu.ITEM_STYLE_ICON)
			&& (iStyle != menu.ITEM_STYLE_CHECKBOX)
			&& (iStyle != menu.ITEM_STYLE_CASCADE)
			&& (iStyle != menu.ITEM_STYLE_DELIMITER))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE, ""),
											"menu.js", "jaswin.menu.MenuItem.constructor");
			
			if(typeof bEnabled != "undefined")
			{
				if(typeof bEnabled != "boolean")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_ENABLED, ""),
												"menu.js", "jaswin.menu.MenuItem.constructor");
			}
			
			if(iStyle != menu.ITEM_STYLE_DELIMITER)
			{
				if(typeof sLabel != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, ""),
												"menu.js", "jaswin.menu.MenuItem.constructor");
				
				if(iStyle == menu.ITEM_STYLE_CHECKBOX)
				{
					if(sGroup)
					{
						if(typeof sGroup != "string")
							throw new jaswin.JaswinError(jaswin.ERR_ALERT,
														jaswin.formatMessage(jaswin.ALERT_TYPE_GROUP, ""),
														"menu.js", "jaswin.menu.MenuItem.constructor");
					}
					
					if(typeof bChecked != "boolean")
						throw new jaswin.JaswinError(jaswin.ERR_ALERT,
													jaswin.formatMessage(jaswin.ALERT_TYPE_CHECKED, ""),
													"menu.js", "jaswin.menu.MenuItem.constructor");
				}
				
				if((iStyle == menu.ITEM_STYLE_IMAGE)
				|| (iStyle == menu.ITEM_STYLE_ICON))
				{
					if(typeof sContents != "string")
						throw new jaswin.JaswinError(jaswin.ERR_ALERT,
													jaswin.formatMessage(jaswin.ALERT_TYPE_CONTENTS, ""),
													"menu.js", "jaswin.menu.MenuItem.constructor");
				}
				
				if(fpClick)
				{
					if(typeof fpClick != "function")
						throw new jaswin.JaswinError(jaswin.ERR_ALERT,
													jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CLICK, ""),
													"menu.js", "jaswin.menu.MenuItem.constructor");
				}
			}
			
			// Set properties
			this.style = iStyle;				// Item style[int]
			this.group = sGroup;				// Group name(for Radio button)[String]
			this.checked = bChecked;			// Checked or not[boolean]
			if(typeof bEnabled == "undefined")
				this.enabled = true;
			else
				this.enabled = bEnabled;		// Enabled or not[boolean]
			this.onClick = fpClick;				// Function on click[function(Argument:MenuItem class)]
			this.element = null;				// Window element[DOM Element]
			this.checkbox = null;				// Checkbox or Radio button
			this.icon = null;					// Icon image or span[DOM Element]
			this.label = null;					// Label element[DOM Element]
			this.arrow = null;					// Arrow icon for cascading[DOM Element]
			this.parent = null;					// Parent menu which this item attached[Menu class]
			this.childMenu = null;				// Child menu for cascading[Menu class]
			
			// Create element
			let oDiv;
			
			this.element = document.createElement("li");
			
			if(this.style == menu.ITEM_STYLE_DELIMITER)
			{
				this.element.className = "menu-item-delimiter";
				oDiv = document.createElement("div");
				oDiv.className = "h-line-inset";
				this.element.appendChild(oDiv);
			}
			else
			{
				this.element.className = "menu-item";
				
				// Make header area
				oDiv = document.createElement("div");
				oDiv.className = "menu-item-header";
				
				if(this.style == menu.ITEM_STYLE_CHECKBOX)
				{
					this.checkbox = document.createElement("span");
					if((this.group) && (this.group != ""))
					{
						// Radio button
						if(this.enabled)
						{
							if(this.checked)
								this.checkbox.className = jaswin.ICON_RADIO_CHECKED;
							else
								this.checkbox.className = jaswin.ICON_RADIO;
						}
						else
						{
							if(this.checked)
								this.checkbox.className = jaswin.ICON_RADIO_CHECKED_DISABLED;
							else
								this.checkbox.className = jaswin.ICON_RADIO;
							this.checkbox.style.cursor = "default";
						}
						this.checkbox.style.left = "7px";
						this.checkbox.style.top = "6px";
					}
					else
					{
						// Checkbox
						if(this.enabled)
						{
							if(this.checked)
								this.checkbox.className = jaswin.ICON_CHECKBOX_CHECKED;
							else
								this.checkbox.className = jaswin.ICON_CHECKBOX;
						}
						else
						{
							if(this.checked)
								this.checkbox.className = jaswin.ICON_CHECKBOX_CHECKED_DISABLED;
							else
								this.checkbox.className = jaswin.ICON_CHECKBOX;
							this.checkbox.style.cursor = "default";
						}
						this.checkbox.style.left = "6px";
						this.checkbox.style.top = "5px";
					}
					oDiv.appendChild(this.checkbox);
				}
				else if(this.style == menu.ITEM_STYLE_IMAGE)
				{
					this.icon = document.createElement("img");
					this.icon.src = sContents;
					if(this.enabled)
						this.icon.className = "menu-item-image";
					else
						this.icon.className = "menu-item-image-disabled";
					oDiv.appendChild(this.icon);
				}
				else if(this.style == menu.ITEM_STYLE_ICON)
				{
					let oSpan = document.createElement("span");
					
					oSpan.className = sContents;
					
					this.icon = document.createElement("span");
					
					if(this.enabled)
						this.icon.className = "menu-item-icon";
					else
						this.icon.className = "menu-item-icon-disabled";
					
					this.icon.appendChild(oSpan);
					oDiv.appendChild(this.icon);
				}
				
				this.element.appendChild(oDiv);
				
				// Make label
				this.label = document.createElement("div");
				if(this.enabled)
					this.label.className = "menu-item-label";
				else
					this.label.className = "menu-item-label-disabled";
				this.label.style.width = this.iWidth - (2 * 2) - 24 - 8 - 2 + "px";	// 2*2:padding 24:header 8:footer 2:border
				this.label.innerHTML = sLabel;
				this.element.appendChild(this.label);
				
				// Make footer
				oDiv = document.createElement("div");
				oDiv.className = "menu-item-footer";
				if(this.style == menu.ITEM_STYLE_CASCADE)
				{
					this.arrow = document.createElement("span");
					if(this.enabled)
						this.arrow.className = jaswin.ICON_TRIANGLE_RIGHT;
					else
						this.arrow.className = jaswin.ICON_TRIANGLE_RIGHT_DISABLED;
					this.arrow.style.top = "6px";
					this.arrow.style.left = "0px";
					oDiv.appendChild(this.arrow);
				}
				this.element.appendChild(oDiv);
				
				if(this.enabled)
				{
					// Bind events
					this.element.addEventListener("mousemove", this.mouseMoveEvent = (e) => {
						this.parent.setItemHover(this);
					}, false);
					this.element.addEventListener("mouseout", this.mouseOutEvent = (e) => {
						this.parent.setItemOut(this);
					}, false);
					
					if(this.style != menu.ITEM_STYLE_CASCADE)
					{
						this.element.addEventListener("click", this.clickEvent = (e) => {
							if(this.onClick)
								this.onClick(this);
							menu.menuCancel();
						}, false);
					}
					if(this.style == menu.ITEM_STYLE_CHECKBOX)
					{
						this.element.addEventListener("mousedown", this.mouseDownEvent = (e) => {
							this.parent.checkItem(this);
						}, false);
					}
				}
			}
		}
		
		//==================================================
		//	Enable this menu item
		//==================================================
		enable()
		{
			if(this.style == menu.ITEM_STYLE_DELIMITER)
				return;
			
			this.enabled = true;
			if(this.checkbox)
			{
				if((this.group) && (this.group != ""))
				{
					// Radio button
					if(this.checked)
						this.checkbox.className = jaswin.ICON_RADIO_CHECKED;
					else
						this.checkbox.className = jaswin.ICON_RADIO;
				}
				else
				{
					// Checkbox
					if(this.checked)
						this.checkbox.className = jaswin.ICON_CHECKBOX_CHECKED;
					else
						this.checkbox.className = jaswin.ICON_CHECKBOX;
				}
				this.checkbox.style.cursor = "pointer";
			}
			else if(this.style == menu.ITEM_STYLE_IMAGE)
				this.icon.className = "menu-item-image";
			else if(this.style == menu.ITEM_STYLE_ICON)
				this.icon.className = "menu-item-icon";
			
			this.label.className = "menu-item-label";
			
			if(this.arrow)
				this.arrow.className = jaswin.ICON_TRIANGLE_RIGHT;
			
			if(!this.mouseMoveEvent)
			{
				this.element.addEventListener("mousemove", this.mouseMoveEvent = (e) => {
					this.parent.setItemHover(this);
				}, false);
			}
			if(!this.mouseOutEvent)
			{
				this.element.addEventListener("mouseout", this.mouseOutEvent = (e) => {
					this.parent.setItemOut(this);
				}, false);
			}
			if(this.style != menu.ITEM_STYLE_CASCADE)
			{
				if(!this.clickEvent)
				{
					this.element.addEventListener("click", this.clickEvent = (e) => {
						if(this.onClick)
							this.onClick(this);
						menu.menuCancel();
					}, false);
				}
			}
			if(this.style == menu.ITEM_STYLE_CHECKBOX)
			{
				if(!this.mouseDownEvent)
				{
					this.element.addEventListener("mousedown", this.mouseDownEvent = (e) => {
						this.parent.checkItem(this);
					}, false);
				}
			}
		}
		
		//==================================================
		//	Disable this menu item
		//==================================================
		disable()
		{
			if(this.style == menu.ITEM_STYLE_DELIMITER)
				return;
			
			this.enabled = false;
			if(this.checkbox)
			{
				if((this.group) && (this.group != ""))
				{
					// Radio button
					if(this.checked)
						this.checkbox.className = jaswin.ICON_RADIO_CHECKED_DISABLED;
					else
						this.checkbox.className = jaswin.ICON_RADIO;
				}
				else
				{
					// Checkbox
					if(this.checked)
						this.checkbox.className = jaswin.ICON_CHECKBOX_CHECKED_DISABLED;
					else
						this.checkbox.className = jaswin.ICON_CHECKBOX;
				}
				this.checkbox.style.cursor = "default";
			}
			else if(this.style == menu.ITEM_STYLE_IMAGE)
				this.icon.className = "menu-item-image-disabled";
			else if(this.style == menu.ITEM_STYLE_ICON)
				this.icon.className = "menu-item-icon-disabled";
			
			this.label.className = "menu-item-label-disabled";
			
			if(this.arrow)
				this.arrow.className = jaswin.ICON_TRIANGLE_RIGHT_DISABLED;
			
			if(this.mouseMoveEvent)
			{
				this.element.removeEventListener("mousemove", this.mouseMoveEvent, false);
				this.mouseMoveEvent = null;
			}
			if(this.mouseOutEvent)
			{
				this.element.removeEventListener("mouseout", this.mouseOutEvent, false);
				this.mouseOutEvent = null;
			}
			
			if((this.style != menu.ITEM_STYLE_CASCADE) && (this.onClick))
			{
				if(this.clickEvent)
				{
					this.element.removeEventListener("click", this.clickEvent, false);
					this.clickEvent = null;
				}
			}
			if(this.style == menu.ITEM_STYLE_CHECKBOX)
			{
				if(this.mouseDownEvent)
				{
					this.element.removeEventListener("mousedown", this.mouseDownEvent, false);
					this.mouseDownEvent = null;
				}
			}
		}
		
		//==================================================
		//	Set click event function
		//==================================================
		setClickEvent(fpClick)
		{
			if(this.style == menu.ITEM_STYLE_DELIMITER)
				return;
			
			if(typeof fpClick != "function")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_ON_CLICK, ""),
											"menu.js", "jaswin.menu.MenuItem.setClickEvent");
			
			this.onClick = fpClick;
			if((this.style != menu.ITEM_STYLE_CASCADE) && (this.onClick))
			{
				if(!this.clickEvent)
				{
					this.element.addEventListener("click", this.clickEvent = (e) => {
						this.onClick(this);
					}, false);
				}
			}
		}
		
		//==================================================
		//	Set child menu
		//==================================================
		setChildMenu(oMenu)
		{
			if(this.style != menu.ITEM_STYLE_CASCADE)
				return;
			
			if((typeof oMenu != "object")
			|| (!oMenu instanceof menu.Menu))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MENU, ""),
											"menu.js", "jaswin.menu.MenuItem.setChildMenu");
			
			this.element.appendChild(oMenu.element);
			this.childMenu = oMenu;
			oMenu.setParent(this, this.parent.depth + 1);
		}
	}
	
	
	//==========================================================
	//
	//	Class		:	Menu
	//
	//	Summary		:	Class of menu
	//
	//==========================================================
	menu.Menu = class extends jaswin.Control
	{
		//==================================================
		// constructor
		//==================================================
		constructor(sID,			// * ID[String]
					iWidth)			// * Width[int]
		{
			// Base class
			let oRect = new jaswin.FixedRect(jaswin.ANCHOR_LEFT, jaswin.ANCHOR_TOP, 0, 0, 0, 0);
			super(false, sID, false, oRect, true, -1);
			
			// Check parameters
			if(typeof iWidth != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_WIDTH, sID),
											"menu.js", "jaswin.menu.Menu.constructor");
			
			// Set properties
			this.width = iWidth;			// Width[int]
			this.items = new Array();		// Menu items[MenuItem class]
			this.depth = 0;					// Depth of cascading[int]
			this.currentIndex = -1;			// Current index of item on focus
			
			// Create element
			this.element = document.createElement("ul");
			this.element.className = "menu";
			this.element.style.display = "none";
			this.element.style.width = iWidth + "px";
			
			if((jaswin.frame) && (jaswin.frame.windowCount))
				this.element.style.zIndex = 100 + jaswin.frame.windowCount;
			else
				this.element.style.zIndex = 100;
		}
		
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy()
		{
			let iCounter;
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if((this.items[iCounter].style == menu.ITEM_STYLE_CASCADE)
				&& (this.items[iCounter].childMenu))
					this.items[iCounter].childMenu.destroy();
			}
			
			delete menu.menus[this.element.id];
			super.destroy();
		}
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent(oParent, iDepth)
		{
			if((typeof oParent != "object")
			|| ((!oParent instanceof jaswin.Container)
			&& (!oParent instanceof menu.MenuItem)
			&& (!oParent instanceof menu.MenuBar)))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_PARENT, this.id),
											"menu.js", "jaswin.menu.Menu.setParent");
			
			this.parent = oParent;
			this.depth = iDepth;
			
			let oParentWnd = jaswin.findParentWindow(this);
			let sID;
			
			if(oParentWnd == global)
				sID = this.id;
			else
			{
				sID = oParentWnd.id + "_" + this.id;
				this.element.style.zIndex = 100 + jaswin.frame.windowCount + oParentWnd.modalLevel;
			}
			
			// Duplication check
			if(document.getElementById(sID))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_DUPLICATION_ID, this.id),
											"menu.js", "jaswin.menu.Menu.setParent");
			
			this.element.id = sID;
			menu.menus[this.element.id] = this;
			oParentWnd[this.id] = this;
		}
		
		//==================================================
		//	Show this menu
		//==================================================
		show()
		{
			if(!this.parent)
				return;
			
			super.show();
			
			this.currentIndex = -1;
			menu.visibleMenus.push(this);
			if(!global.menuCancelEvent)
			{
				global.element.addEventListener("click", global.menuCancelEvent = (e) => {
					let oMenuItem = this.findEventItem(e);
					if((oMenuItem) && (oMenuItem.style == menu.ITEM_STYLE_CASCADE))
						return false;
					menu.menuCancel();
				}, false);
			}
			if(!this.keyEvent)
			{
				this.element.addEventListener("menuKey", this.keyEvent = (e) => {
					this.menuKey(e);
				}, false);
			}
		}
		
		//==================================================
		//	Hide this menu
		//==================================================
		hide()
		{
			if(!this.parent)
				return;
			
			super.hide();
			
			let iCounter;
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(!this.items[iCounter].enabled)
					continue;
				if(this.items[iCounter].style == menu.ITEM_STYLE_DELIMITER)
					continue;
				
				this.items[iCounter].element.className = "menu-item";
				if(this.items[iCounter].style == menu.ITEM_STYLE_CHECKBOX)
				{
					if((this.items[iCounter].group)
					&& (this.items[iCounter].group != ""))
					{
						// Radio button item
						if(!this.items[iCounter].checked)
							this.items[iCounter].checkbox.className = jaswin.ICON_RADIO;
					}
					else
					{
						// Checkbox item
						if(!this.items[iCounter].checked)
							this.items[iCounter].checkbox.className = jaswin.ICON_CHECKBOX;
					}
				}
				else if(this.items[iCounter].style == menu.ITEM_STYLE_CASCADE)
				{
					if(this.items[iCounter].childMenu)
						this.items[iCounter].childMenu.hide();
				}
			}
			
			for(iCounter = 0; iCounter < menu.visibleMenus.length; iCounter++)
			{
				if(menu.visibleMenus[iCounter] == this)
				{
					menu.visibleMenus.splice(iCounter, 1);
					break;
				}
			}
			this.currentIndex = -1;
		}
		
		//==================================================
		//	Get height of this menu
		//==================================================
		getHeight()
		{
			let iCounter;
			let iHeight = 0;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].style == menu.ITEM_STYLE_DELIMITER)
					iHeight += 4;		// Delimiter
				else
					iHeight += 20 + 2;	// 20:Item height 2:border
			}
			
			return iHeight;
		}
		
		//==================================================
		//	Show this menu as a context menu
		//==================================================
		showContext(e)
		{
			let oRect;
			let iGlobalWidth;
			let iGlobalHeight;
			let iMenuHeight;
			
			if(!this.parent)
				return;
			if(!this.parent.enabled)
				return;
			
			menu.menuCancel();
			
			if((e.clientX == 0) && (e.clientY == 0))
			{
				this.element.style.left = "0px";
				this.element.style.top = "0px";
			}
			else
			{
				iGlobalWidth = global.element.scrollWidth;
				iGlobalHeight = global.element.scrollHeight;
				
				oRect = this.element.parentNode.getBoundingClientRect();
				
				if(e.clientX + this.width + 2 >= iGlobalWidth)
					this.element.style.left = e.clientX - this.width - oRect.left + "px";
				else
					this.element.style.left = e.clientX - oRect.left + "px";
				
				iMenuHeight = this.getHeight();
				if(e.clientY + iMenuHeight + 2 >= iGlobalHeight)
					this.element.style.top = e.clientY  - iMenuHeight - oRect.top + "px";
				else
					this.element.style.top = e.clientY - oRect.top + "px";
			}
			
			this.currentIndex = -1;
			this.show();
		}
		
		//==================================================
		//	Show this menu as a child menu
		//==================================================
		showChildMenu()
		{
			let oRect;
			let iGlobalWidth;
			let iGlobalHeight;
			let iParentWidth;
			let iMenuHeight;
			
			if(!this.parent)
				return;
			
			iGlobalWidth = global.element.scrollWidth;
			iGlobalHeight = global.element.scrollHeight;
			
			oRect = this.element.parentNode.getBoundingClientRect();
			iParentWidth = this.parent.element.offsetWidth;
			iMenuHeight = this.getHeight();
			
			if(oRect.left + iParentWidth + 2 + this.width + 2 >= iGlobalWidth)
				this.element.style.left = - this.width + "px";
			else
				this.element.style.left = iParentWidth - 2 + "px";
			
			if(iMenuHeight + 2 >= iGlobalHeight)
				this.element.style.top = - oRect.top + "px";
			else if(oRect.top + iMenuHeight + 2 + 4 >= iGlobalHeight)
				this.element.style.top = iGlobalHeight - (oRect.top + iMenuHeight + 2 + 4) + "px";
			else
				this.element.style.top = "0px";
			
			this.currentIndex = -1;
			this.show();
		}
		
		//==================================================
		//	Add menu item to this menu
		//==================================================
		addItem(oItem)
		{
			if((typeof oItem != "object")
			|| (!oItem instanceof menu.MenuItem))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MENU_ITEM, this.id),
											"menu.js", "jaswin.menu.Menu.addItem");
			
			this.element.appendChild(oItem.element);
			this.items.push(oItem);
			oItem.parent = this;
		}
		
		//==================================================
		//	Remove menu item from this menu
		//==================================================
		removeItem(iIndex)
		{
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"menu.js", "jaswin.menu.Menu.removeItem");
			
			if((iIndex < 0) || (iIndex >= this.items.length))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_INDEX, this.id),
											"menu.js", "jaswin.menu.Menu.removeItem");
			
			this.element.removeChild(this.items[iIndex].element);
			this.items.splice(iIndex, 1);
		}
		
		//==================================================
		//	Clear menu item from this menu
		//==================================================
		clearItem()
		{
			while(true)
			{
				if(this.items.length == 0)
					break;
				
				this.removeItem(0);
			}
		}
		
		//==================================================
		//	Set item of this menu hover
		//==================================================
		setItemHover(oItem)
		{
			let iCounter;
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].style == menu.ITEM_STYLE_DELIMITER)
					continue;
				
				if(!this.items[iCounter].enabled)
					continue;
				
				if(this.items[iCounter] == oItem)
				{
					this.currentIndex = iCounter;
					this.items[iCounter].element.className = "menu-item-focus";
					if(this.items[iCounter].style == menu.ITEM_STYLE_CHECKBOX)
					{
						if((this.items[iCounter].group)
						&& (this.items[iCounter].group != ""))
						{
							// Radio button item
							if(!this.items[iCounter].checked)
								this.items[iCounter].checkbox.className = jaswin.ICON_RADIO_HOVER;
						}
						else
						{
							// Checkbox item
							if(!this.items[iCounter].checked)
								this.items[iCounter].checkbox.className = jaswin.ICON_CHECKBOX_HOVER;
						}
					}
					else if(this.items[iCounter].style == menu.ITEM_STYLE_CASCADE)
					{
						if(this.items[iCounter].childMenu)
						{
							// Show child menu
							if(!this.items[iCounter].childMenu.isVisible())
							{
								if((jaswin.frame) && (jaswin.frame.windowCount))
								{
									let oParentWnd = jaswin.findParentWindow(this);
									if(oParentWnd != global)
										this.items[iCounter].childMenu.element.style.zIndex = 100 + this.items[iCounter].childMenu.depth + jaswin.frame.windowCount + oParentWnd.modalLevel;
									else
										this.items[iCounter].childMenu.element.style.zIndex = 100 + this.items[iCounter].childMenu.depth + jaswin.frame.windowCount;
								}
								else
									this.items[iCounter].childMenu.element.style.zIndex = 100 + this.items[iCounter].childMenu.depth;
								this.items[iCounter].childMenu.showChildMenu();
							}
						}
					}
				}
				else
				{
					this.items[iCounter].element.className = "menu-item";
					
					if(this.items[iCounter].style == menu.ITEM_STYLE_CHECKBOX)
					{
						if((this.items[iCounter].group)
						&& (this.items[iCounter].group != ""))
						{
							// Radio button item
							if(!this.items[iCounter].checked)
								this.items[iCounter].checkbox.className = jaswin.ICON_RADIO;
						}
						else
						{
							// Checkbox item
							if(!this.items[iCounter].checked)
								this.items[iCounter].checkbox.className = jaswin.ICON_CHECKBOX;
						}
					}
					else if(this.items[iCounter].style == menu.ITEM_STYLE_CASCADE)
					{
						if(this.items[iCounter].childMenu)
						{
							// Hide child menu
							this.items[iCounter].childMenu.hide();
							this.items[iCounter].element.className = "menu-item";
						}
					}
				}
			}
		}
		
		//==================================================
		//	Set item of this menu mouse out
		//==================================================
		setItemOut(oItem)
		{
			let iCounter;
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].style == menu.ITEM_STYLE_DELIMITER)
					continue;
				
				if(this.items[iCounter] == oItem)
				{
					if(!this.items[iCounter].enabled)
						return;
					
					this.currentIndex = -1;
					
					if(this.items[iCounter].style == menu.ITEM_STYLE_CASCADE)
					{
						if(this.items[iCounter].childMenu)
						{
							if(!this.items[iCounter].childMenu.isVisible())
								this.items[iCounter].element.className = "menu-item";
						}
						else
							this.items[iCounter].element.className = "menu-item";
					}
					else
						this.items[iCounter].element.className = "menu-item";
					
					if(this.items[iCounter].style == menu.ITEM_STYLE_CHECKBOX)
					{
						if((this.items[iCounter].group)
						&& (this.items[iCounter].group != ""))
						{
							// Radio button item
							if(!this.items[iCounter].checked)
								this.items[iCounter].checkbox.className = jaswin.ICON_RADIO;
						}
						else
						{
							// Checkbox item
							if(!this.items[iCounter].checked)
								this.items[iCounter].checkbox.className = jaswin.ICON_CHECKBOX;
						}
					}
					
					break;
				}
			}
		}
		
		//==================================================
		//	Set focus to a menu item
		//==================================================
		setItemFocus(iIndex)
		{
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"menu.js", "jaswin.menu.Menu.setItemFocus");
			
			let iCounter = 0;
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].style == menu.ITEM_STYLE_DELIMITER)
					continue;
				
				if(iCounter == iIndex)
				{
					if(!this.items[iCounter].enabled)
						continue;
					
					this.currentIndex = iCounter;
					this.items[iCounter].element.className = "menu-item-focus";
					
					if(this.items[iCounter].style == menu.ITEM_STYLE_CHECKBOX)
					{
						if((this.items[iCounter].group)
						&& (this.items[iCounter].group != ""))
						{
							// Radio button item
							if(!this.items[iCounter].checked)
								this.items[iCounter].checkbox.className = jaswin.ICON_RADIO_HOVER;
						}
						else
						{
							// Checkbox item
							if(!this.items[iCounter].checked)
								this.items[iCounter].checkbox.className = jaswin.ICON_CHECKBOX_HOVER;
						}
					}
					else if(this.items[iCounter].style == menu.ITEM_STYLE_CASCADE)
					{
						if(this.items[iCounter].childMenu)
						{
							// Show child menu
							if(!this.items[iCounter].childMenu.isVisible())
							{
								if((jaswin.frame) && (jaswin.frame.windowCount))
								{
									let oParentWnd = jaswin.findParentWindow(this);
									if(oParentWnd != global)
										this.items[iCounter].childMenu.element.style.zIndex = 100 + this.items[iCounter].childMenu.depth + jaswin.frame.windowCount +  + oParentWnd.modalLevel;
									else
										this.items[iCounter].childMenu.element.style.zIndex = 100 + this.items[iCounter].childMenu.depth + jaswin.frame.windowCount;
								}
								else
									this.items[iCounter].childMenu.element.style.zIndex = 100 + this.items[iCounter].childMenu.depth;
								this.items[iCounter].childMenu.showChildMenu();
							}
						}
					}
				}
				else
				{
					if(!this.items[iCounter].enabled)
						continue;
					
					this.items[iCounter].element.className = "menu-item";
					
					if(this.items[iCounter].style == menu.ITEM_STYLE_CHECKBOX)
					{
						if((this.items[iCounter].group)
						&& (this.items[iCounter].group != ""))
						{
							// Radio button item
							if(!this.items[iCounter].checked)
								this.items[iCounter].checkbox.className = jaswin.ICON_RADIO;
						}
						else
						{
							// Checkbox item
							if(!this.items[iCounter].checked)
								this.items[iCounter].checkbox.className = jaswin.ICON_CHECKBOX;
						}
					}
					else if(this.items[iCounter].style == menu.ITEM_STYLE_CASCADE)
					{
						if(this.items[iCounter].childMenu)
						{
							// Hide child menu
							this.items[iCounter].childMenu.hide();
							this.items[iCounter].element.className = "menu-item";
						}
					}
				}
			}
		}
		
		//==================================================
		//	Find event item of this menu by label
		//==================================================
		findItem(sLabel)
		{
			if(typeof sLabel != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, this.id),
											"menu.js", "jaswin.menu.Menu.findItem");
			
			let iCounter;
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].style == menu.ITEM_STYLE_DELIMITER)
					continue;
				if(this.items[iCounter].label.innerHTML == sLabel)
					return this.items[iCounter];
			}
			
			return null;
		}
		
		//==================================================
		//	Find event item of this menu
		//==================================================
		findEventItem(e)
		{
			let oElement = e.target;
			let iCounter = 0;
			let oTargetItem = null;
			
			if(!oElement)
				return null;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if(this.items[iCounter].element.contains(oElement))
				{
					oTargetItem = this.items[iCounter];
					break;
				}
			}
			return oTargetItem;
		}
		
		//==================================================
		//	Check item of this menu
		//==================================================
		checkItem(oItem)
		{
			if((oItem.group)
			&& (oItem.group != ""))
			{
				// Radio button item
				oItem.checked = true;
				oItem.checkbox.className = jaswin.ICON_RADIO_CHECKED;
				
				let iCounter = 0;
				for(iCounter = 0; iCounter < this.items.length; iCounter++)
				{
					if(this.items[iCounter] == oItem)
						continue;
					
					if((this.items[iCounter].style == menu.ITEM_STYLE_CHECKBOX)
					&& (this.items[iCounter].group)
					&& (this.items[iCounter].group == oItem.group))
					{
						this.items[iCounter].checked = false;
						this.items[iCounter].checkbox.className = jaswin.ICON_RADIO;
					}
				}
			}
			else
			{
				// Checkbox item
				if(oItem.checked)
				{
					oItem.checked = false;
					oItem.checkbox.className = jaswin.ICON_CHECKBOX;
				}
				else
				{
					oItem.checked = true;
					oItem.checkbox.className = jaswin.ICON_CHECKBOX_CHECKED;
				}
			}
		}
		
		//==================================================
		//	Check item of this menu by index
		//==================================================
		checkItemByIndex(iIndex)
		{
			if(this.items[iIndex].style != menu.ITEM_STYLE_CHECKBOX)
				return;
			
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"menu.js", "jaswin.menu.Menu.checkItemByIndex");
			if((iIndex < 0) || (iIndex >= this.items.length))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_INDEX, this.id),
											"menu.js", "jaswin.menu.Menu.checkItemByIndex");
			
			this.checkItem(this.items[iIndex]);
			
			if(this.items[iIndex].onClick)
				this.items[iIndex].onClick(this.items[iIndex]);
		}
		
		//==================================================
		//	Enter key item of this menu
		//==================================================
		enterItem(iIndex)
		{
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"menu.js", "jaswin.menu.Menu.enterItem");
			if((iIndex < 0) || (iIndex >= this.items.length))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_INDEX, this.id),
											"menu.js", "jaswin.menu.Menu.enterItem");
			
			if(this.items[iIndex].style == menu.ITEM_STYLE_CHECKBOX)
				this.checkItem(this.items[iIndex]);
			
			if((this.items[iIndex].style != menu.ITEM_STYLE_CASCADE)
			&& (this.items[iIndex].onClick))
				this.items[iIndex].onClick(this.items[iIndex]);
			
			menu.menuCancel();
		}
		
		//==================================================
		//	Keydown to this menu
		//==================================================
		menuKey(e)
		{
			let iIndex = this.currentIndex;
			let iCounter;
			
			if(e.detail == 40)
			{
				// down
				if(this.currentIndex == this.items.length -1)
					return;
				
				for(iCounter = this.currentIndex + 1; iCounter < this.items.length; iCounter++)
				{
					if(this.items[iCounter].style == menu.ITEM_STYLE_DELIMITER)
						continue;
					if(!this.items[iCounter].enabled)
						continue;
					
					iIndex = iCounter;
					break;
				}
				
				if(iIndex == this.currentIndex)
					return;
				this.setItemFocus(iIndex);
			}
			else if(e.detail == 38)
			{
				// up
				if(this.currentIndex == 0)
					return;
				
				for(iCounter = this.currentIndex - 1; iCounter >= 0; iCounter--)
				{
					if(this.items[iCounter].style == menu.ITEM_STYLE_DELIMITER)
						continue;
					if(!this.items[iCounter].enabled)
						continue;
					
					iIndex = iCounter;
					break;
				}
				
				if(iIndex == this.currentIndex)
					return;
				this.setItemFocus(iIndex);
			}
			else if(e.detail == 39)
			{
				// right
				if((this.currentIndex == -1)
				|| (this.items[this.currentIndex].style != menu.ITEM_STYLE_CASCADE))
				{
					if(this.parent instanceof menu.MenuBar)
					{
						for(iCounter = 0; iCounter < this.parent.items.length; iCounter++)
						{
							if(this.parent.items[iCounter].menu == this)
								break;
						}
						if(iCounter < this.parent.items.length - 1)
							this.parent.items[iCounter + 1].click();
					}
					return;
				}
				
				if(this.currentIndex == -1)
					return;
				
				if((this.items[this.currentIndex].enabled)
				&& (this.items[this.currentIndex].childMenu))
				{
					if(!this.items[this.currentIndex].childMenu.isVisible())
						this.items[this.currentIndex].childMenu.show();
				}
			}
			else if(e.detail == 37)
			{
				// left
				if(this.parent instanceof menu.MenuBar)
				{
					for(iCounter = 0; iCounter < this.parent.items.length; iCounter++)
					{
						if(this.parent.items[iCounter].menu == this)
							break;
					}
					if(iCounter > 0)
						this.parent.items[iCounter - 1].click();
				}
				else if(this.parent instanceof menu.MenuItem)
					this.hide();
			}
			else if(e.detail == 13)
			{
				// enter
				if(this.currentIndex != -1)
				{
					this.enterItem(this.currentIndex);
					
					if(this.parent instanceof menu.MenuBar)
						this.parent.setInactive();
				}
			}
			else if(e.detail == 27)
			{
				// escape
				menu.menuCancel();
			}
			else if(e.detail == 32)
			{
				// space
				if(this.currentIndex != -1)
					this.checkItemByIndex(this.currentIndex);
			}
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	menuCancel
	//
	//	Summary		:	Cancel menu
	//
	//==========================================================
	menu.menuCancel = function()
	{
		for(let sKey in menu.menus)
			menu.menus[sKey].hide();
		
		menu.visibleMenus = new Array();
		
		if(global.menuCancelEvent)
		{
			global.element.removeEventListener("click", global.menuCancelEvent, false);
			global.menuCancelEvent = null;
		}
		
		menu.menuBarItemCancel();
	}
	
	
	//==========================================================
	//
	//	Class		:	MenuBarItem
	//
	//	Summary		:	Class of window menu bar item
	//
	//==========================================================
	menu.MenuBarItem = class
	{
		//==================================================
		// constructor
		//==================================================
		constructor(sLabel,			// * Label[String]
					oMenu,			// * Menu which this item attached[Menu class]
					sShortcut)		// Shortcut[Char]
		{
			// Check parameters
			let sShortcutChar = null;
			
			if(typeof sLabel != "string")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_LABEL, sID),
											"menu.js", "jaswin.menu.MenuBarItem.constructor");
			
			if((typeof oMenu != "object")
			|| (!oMenu instanceof menu.Menu))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MENU, sID),
											"menu.js", "jaswin.menu.MenuBarItem.constructor");
			
			if(sShortcut)
			{
				if(typeof sShortcut != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_SHORTCUT, sID),
												"menu.js", "jaswin.menu.MenuBarItem.constructor");
				if(sShortcut.length >= 0)
				{
					sShortcutChar = sShortcut.charAt(0);
					if(!sShortcutChar.match(/[a-zA-Z]/))
						throw new jaswin.JaswinError(jaswin.ERR_ALERT,
													jaswin.formatMessage(jaswin.ALERT_VALUE_SHORTCUT, sID),
													"menu.js", "jaswin.menu.MenuBarItem.constructor");
					
					sShortcutChar = sShortcutChar.toUpperCase();
				}
			}
			
			// Set properties
			this.status = menu.BAR_ITEM_STATUS_DEFAULT;		// Status[int]
			this.shortcut = sShortcutChar;					// Shortcut character[String]
			this.element = null;							// Window element[DOM Element]
			this.menu = oMenu;								// Menu[Menu class]
			this.parent = null;								// Parent menu bar[MenuBar class]
			
			// Create element
			this.element = document.createElement("span");
			this.element.innerHTML = sLabel;
			
			if(sShortcutChar)
			{
				this.element.innerHTML += "(<em class=\"menu-bar-item-shortcut\">"
										+ sShortcutChar
										+ "</em>)";
			}
			
			// Event binding
			this.element.className = "menu-bar-item";
			this.element.addEventListener("mousemove", this.mouseMoveEvent = (e) => {
				this.mouseOver();
			}, false);
			this.element.addEventListener("click", this.clickEvent = (e) => {
				this.click();
				e.stopPropagation();
			}, false);
		}
		
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy()
		{
			if(this.menu.element)
				this.menu.destroy();
			this.element.parentNode.removeChild(this.element);
			
			return true;
		}
		
		//==================================================
		//	Set active
		//==================================================
		setActive()
		{
			let iScroll = this.element.parentNode.parentNode.scrollLeft;
			let iLeft = this.element.offsetLeft + 2 - iScroll;
			
			if(iLeft < 0)
				iLeft = 0;
			this.menu.element.style.left = iLeft + "px";
			this.menu.element.style.top = "20px";
			this.menu.show();
			this.element.className = "menu-bar-item-active";
			this.status = menu.BAR_ITEM_STATUS_ACTIVE;
		}
		
		//==================================================
		//	Set inctive
		//==================================================
		setInactive()
		{
			this.element.className = "menu-bar-item";
			this.menu.hide();
			this.status = menu.BAR_ITEM_STATUS_DEFAULT;
		}
		
		//==================================================
		//	Mouse over this menu bar item
		//==================================================
		mouseOver()
		{
			let iCounter;
			let bVisible;
			let iIndex;
			
			for(iCounter = 0; iCounter < this.parent.items.length; iCounter++)
			{
				if(this.parent.items[iCounter].menu.isVisible())
				{
					if(this.parent.items[iCounter] == this)
						return;
					
					bVisible = true;
					iIndex = iCounter;
					break;
				}
			}
			
			if(bVisible)
			{
				this.parent.items[iIndex].setInactive();
				this.setActive();
			}
		}
		
		//==================================================
		//	Click this menu bar item
		//==================================================
		click()
		{
			if(!this.menu)
				return;
			
			if(this.menu.isVisible())
			{
				this.element.className = "menu-bar-item";
				this.menu.hide();
				this.status = menu.BAR_ITEM_STATUS_DEFAULT;
			}
			else
			{
				menu.menuCancel();
				
				let iCounter;
				for(iCounter = 0; iCounter < this.parent.items.length; iCounter++)
					this.parent.items[iCounter].setInactive();
				
				this.setActive();
				
				return;
			}
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	menuBarItemCancel
	//
	//	Summary		:	Event handler of cancel a menu bar
	//
	//==========================================================
	menu.menuBarItemCancel = function()
	{
		let oMenuBar = null;
		
		if(jaswin.frame)
		{
			if(jaswin.currentWindow)
				oMenuBar = jaswin.currentWindow.menu;
			else
				oMenuBar = global.menu;
		}
		else
			oMenuBar = global.menu;
		
		if(oMenuBar)
			oMenuBar.setInactive();
	}
	
	
	//==========================================================
	//
	//	Class		:	MenuBar
	//
	//	Summary		:	Class of window menu bar
	//
	//==========================================================
	menu.MenuBar = class extends jaswin.Control
	{
		//==================================================
		// constructor
		//==================================================
		constructor()
		{
			// Base class
			let oRect = new jaswin.FixedRect(jaswin.ANCHOR_LEFT, jaswin.ANCHOR_TOP, 0, 0, 0, 0);
			super(false, "MenuBar", false, oRect, true, -1);
			
			// Set properties
			this.items = new Array();		// Header items[MenuBarItem class]
			this.headerArea = null;			// Area for header items[DOM Element]
			this.menuArea = null;			// Area for menus[DOM Element]
			
			// Create element
			this.element = document.createElement("div");
			this.element.className = "menu-bar";
			this.headerArea = document.createElement("div");
			this.headerArea.className = "menu-bar-container";
			this.headerInner = document.createElement("div");
			this.headerInner.className = "menu-bar-inner";
			this.headerArea.appendChild(this.headerInner);
			this.menuArea = document.createElement("div");
			
			this.element.appendChild(this.headerArea);
			this.element.appendChild(this.menuArea);
			
			// Scroll button
			if(jaswin.button)
			{
				let oButtonRect = new jaswin.FixedRect(jaswin.ANCHOR_LEFT, jaswin.ANCHOR_TOP,
														0, 2, 12, 18);
				this.scrollLeftButton = new jaswin.button.Button("BtnMenuBar_Left", oButtonRect,
																jaswin.button.STYLE_DEFAULT | jaswin.button.STYLE_ICON,
																true,
																jaswin.ICON_TRIANGLE_LEFT, null,
																0, menuBarScrollLeft);
				this.scrollLeftButton.hide();
				this.scrollLeftButton.content.childNodes[0].style.left = "-4px";
				this.element.appendChild(this.scrollLeftButton.element);
				
				oButtonRect = new jaswin.FixedRect(jaswin.ANCHOR_RIGHT, jaswin.ANCHOR_TOP,
													0, 2, 12, 18);
				this.scrollRightButton = new jaswin.button.Button("BtnMenuBar_Right", oButtonRect,
																jaswin.button.STYLE_DEFAULT | jaswin.button.STYLE_ICON,
																true,
																jaswin.ICON_TRIANGLE_RIGHT, null,
																0, menuBarScrollRight);
				this.scrollRightButton.hide();
				this.scrollRightButton.content.childNodes[0].style.left = "2px";
				this.element.appendChild(this.scrollRightButton.element);
				
				this.element.addEventListener("mousemove", this.menuBarMouseMove = (e) => {
					this.hover();
				}, false);
				this.element.addEventListener("mouseout", this.menuBarMouseOut = (e) => {
					this.mouseOut(e);
				}, false);
			}
		}
		
		//==================================================
		//	Delete DOM element
		//==================================================
		destroy()
		{
			let iCounter;
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
				this.items[iCounter].destroy();
			
			if(this.scrollLeftButton)
				this.scrollLeftButton.destroy();
			if(this.scrollRightButton)
				this.scrollRightButton.destroy();
			
			this.element.parentNode.removeChild(this.element);
		}
		
		//==================================================
		//	Set parent class
		//==================================================
		setParent(oParent)
		{
			if((typeof oParent != "object")
			|| (!oParent instanceof jaswin.BaseWindow))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_PARENT, ""),
											"menu.js", "jaswin.menu.MenuBar.setParent");
			
			this.parent = oParent;
			if(this.scrollLeftButton)
				this.scrollLeftButton.setParent(this);
			if(this.scrollRightButton)
				this.scrollRightButton.setParent(this);
			
			// Adjust index height
			let iCounter;
			let iHeight;
			let iPadding;
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				this.items[iCounter].menu.setParent(this, 0);
				iHeight = this.items[iCounter].element.offsetHeight;
				iPadding = Math.floor((20 - iHeight) / 2);
				if(iPadding > 0)
				{
					this.items[iCounter].element.style.paddingTop = iPadding + "px";
					this.items[iCounter].element.style.paddingBottom = iPadding + "px";
				}
			}
			
			// Set index scroll area
			let iInnerWidth = 0;
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
				iInnerWidth += this.items[iCounter].element.offsetWidth;
			
			this.headerInner.style.width = iInnerWidth + 1 + "px";
		}
		
		//==================================================
		//	Add menu to this menu bar
		//==================================================
		addMenu(oMenuBarItem)
		{
			if((typeof oMenuBarItem != "object")
			|| (!oMenuBarItem instanceof menu.MenuBarItem))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_MENU_BAR_ITEM, this.id),
											"menu.js", "jaswin.menu.MenuBar.addMenu");
			
			this.items.push(oMenuBarItem);
			oMenuBarItem.parent = this;
			this.headerInner.appendChild(oMenuBarItem.element);
			this.menuArea.appendChild(oMenuBarItem.menu.element);
			
			if(this.parent)
			{
				// Set parent to menu
				oMenuBarItem.menu.setParent(this, 0);
				
				// Set index scroll button
				let iInnerWidth = 0;
				let iCounter;
				
				for(iCounter = 0; iCounter < this.items.length; iCounter++)
					iInnerWidth += this.items[iCounter].element.offsetWidth;
				
				this.headerInner.style.width = iInnerWidth + 1 + "px";
			}
		}
		
		//==================================================
		//	Remove menu from this menu bar
		//==================================================
		removeMenu(iIndex)
		{
			if(typeof iIndex != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_INDEX, this.id),
											"menu.js", "jaswin.menu.MenuBar.removeMenu");
			
			if((iIndex < 0) || (iIndex >= this.items.length))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_INDEX, this.id),
											"menu.js", "jaswin.menu.MenuBar.removeMenu");
			
			let oItem = this.items[iIndex];
			this.items.splice(iIndex, 1);
			oItem.destroy();
		}
		
		//==================================================
		//	Short cut key to a menu bar item
		//==================================================
		shortCut(sChar)
		{
			let iCounter;
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
			{
				if((this.items[iCounter].shortcut)
				&& (this.items[iCounter].shortcut == sChar))
				{
					this.items[iCounter].click();
					return;
				}
			}
		}
		
		//==================================================
		//	Mouse move this menu bar
		//==================================================
		hover()
		{
			if((!this.scrollLeftButton)
			|| (!this.scrollRightButton))
				return;
			
			let iLastScroll = 0;
			let iCurScroll = this.headerArea.scrollLeft;
			let iIndexWidth = this.element.offsetWidth - 4;
			let iInnerWidth = 0;
			let iCounter;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
				iInnerWidth += this.items[iCounter].element.offsetWidth;
			iLastScroll = iInnerWidth - iIndexWidth;
			
			if(iCurScroll)
				this.scrollLeftButton.show();
			if(iInnerWidth > iIndexWidth)
			{
				if(iCurScroll != iLastScroll)
					this.scrollRightButton.show();
			}
		}
		
		//==================================================
		//	Mouse out this menu bar
		//==================================================
		mouseOut(e)
		{
			if((!this.scrollLeftButton)
			|| (!this.scrollRightButton))
				return;
			
			let oElement = e.target;
			let oRect = this.element.getBoundingClientRect();
			
			if((e.clientX < oRect.left)
			|| (e.clientX > oRect.right)
			|| (e.clientY < oRect.top)
			|| (e.clientY > oRect.bottom)
			|| (this.headerInner.contains(oElement)))
			{
				this.scrollLeftButton.hide();
				this.scrollRightButton.hide();
			}
		}
		
		//==================================================
		//	Scroll left this menu bar
		//==================================================
		scrollLeft()
		{
			let iCurScroll = this.headerArea.scrollLeft;
			
			if(iCurScroll - 64 < 0)
				this.headerArea.scrollLeft = 0;
			else
				this.headerArea.scrollLeft = iCurScroll - 64;
		}
		
		//==================================================
		//	Scroll right this menu bar
		//==================================================
		scrollRight()
		{
			let iLastScroll = 0;
			let iCurScroll = this.headerArea.scrollLeft;
			let iIndexWidth = this.element.offsetWidth - 4;
			let iInnerWidth = 0;
			let iCounter;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
				iInnerWidth += this.items[iCounter].element.offsetWidth;
			iLastScroll = iInnerWidth - iIndexWidth;
			
			if(iCurScroll + 64 > iLastScroll)
				this.headerArea.scrollLeft = iLastScroll;
			else
				this.headerArea.scrollLeft += 64;
		}
		
		//==================================================
		//	Set inctive
		//==================================================
		setInactive()
		{
			let iCounter;
			
			for(iCounter = 0; iCounter < this.items.length; iCounter++)
				this.items[iCounter].setInactive();
		}
	}
	
	
	//==========================================================
	//
	//	Function	:	menuBarScrollLeft
	//
	//	Summary		:	Event handler of scroll left a header
	//
	//==========================================================
	function menuBarScrollLeft(oBtn)
	{
		oBtn.parent.scrollLeft();
	}
	
	
	//==========================================================
	//
	//	Function	:	menuBarScrollRight
	//
	//	Summary		:	Event handler of scroll right a header
	//
	//==========================================================
	function menuBarScrollRight(oBtn)
	{
		oBtn.parent.scrollRight();
	}
	
}(jaswin.menu = jaswin.menu || {}));
