//==================================================================
//
//	File name	:	progressbar.js
//
//	Summary		:	ProgressBar class
//
//	Ceated		:	2018/12/05
//	Written by	:	Yuji Iihara
//	Copyright	:	Softinc.
//
//==================================================================
"use strict";

(function (progressbar) {
	//==========================================================
	//	Global
	//==========================================================
	// Const
	progressbar.STYLE_DEFAULT = 0;
	progressbar.STYLE_RATIO = 1;
	progressbar.STYLE_DETAIL = 2;
	
	//==========================================================
	//
	//	Class		:	ProgressBar
	//
	//	Summary		:	Class of progressbar
	//
	//==========================================================
	progressbar.ProgressBar = class extends jaswin.Control
	{
		//==================================================
		// constructor
		//==================================================
		constructor(sID,			// * ID[String]
					oRect,			// * Size and position[FixedRect]
					iStyle,			// * Item style[int]
					bEnabled,		// * Enabled or not[boolean]
					fTotal,			// * Total value[float or int]
					iDigit,			// * Digit number[int]
					sUnit)
		{
			// Base class
			super(false, sID, false, oRect, bEnabled, -1);
			
			// Check parameters
			if(typeof iStyle != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_STYLE, sID),
											"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
			if((iStyle != progressbar.STYLE_DEFAULT)
			&& (iStyle != progressbar.STYLE_RATIO)
			&& (iStyle != progressbar.STYLE_DETAIL))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_STYLE, sID),
											"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
			
			if(typeof fTotal != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOTAL, sID),
											"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
			
			if(typeof iDigit != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_DIGIT, sID),
											"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
			if((iDigit < 0) || (iDigit > 20))
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_VALUE_DIGIT, sID),
											"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
			
			if(iStyle == progressbar.STYLE_DETAIL)
			{
				if(!sUnit)
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_MUST_UNIT, sID),
												"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
			}
			if(sUnit)
			{
				if(typeof sUnit != "string")
					throw new jaswin.JaswinError(jaswin.ERR_ALERT,
												jaswin.formatMessage(jaswin.ALERT_TYPE_UNIT, sID),
												"progressbar.js", "jaswin.progressbar.ProgressBar.constructor");
			}
			
			// Set properties
			this.style = iStyle;			// ProgressBar style[int]
			this.unit = sUnit;				// Unit[string]
			this.total = fTotal;			// Total value[float or int]
			this.value = 0;					// Current value[float or int]
			this.digit = iDigit;			// Digit number[int]
			this.complete = null;			// Element of completed[DOM Element]
			this.label = null;				// Label element for ratio or detail[DOM Element]
			
			// Create element
			this.element = document.createElement("div");
			this.element.className = "progress-bar";
			
			if(this.rect.anchorX == jaswin.ANCHOR_RIGHT)
				this.element.style.right = this.rect.posX + "px";
			else
				this.element.style.left = this.rect.posX + "px";
			
			if(this.rect.anchorY == jaswin.ANCHOR_BOTTOM)
				this.element.style.bottom = this.rect.posY + "px";
			else
				this.element.style.top = this.rect.posY + "px";
			
			this.element.style.width = this.rect.width - 2 + "px";		// 2:border
			this.element.style.height = this.rect.height - 2 + "px";	// 2:border
			
			this.complete = document.createElement("div");
			this.complete.className = "progress-on";
			this.complete.style.width = "0%";
			this.element.appendChild(this.complete);
			
			if(this.style != progressbar.STYLE_DEFAULT)
			{
				this.label = document.createElement("span");
				this.label.className = "progress-text";
				this.label.innerHTML = "0%";
				
				if(this.rect.height > 20)
					this.label.style.marginTop = (this.rect.height - 20) / 2 + "px";
				
				if(this.style == progressbar.STYLE_DETAIL)
				{
					this.label.innerHTML += " (" + this.value.toFixed(this.digit)
											+ "/" + this.total.toFixed(this.digit) + this.unit + ")";
				}
				this.element.appendChild(this.label);
			}
		}
		
		//==================================================
		//	Enable this progressbar
		//==================================================
		enable(bParent)
		{
			super.enable(bParent);
			
			if(this.enabled)
			{
				this.complete.className = "progress-on";
				if(this.label)
					this.label.className = "progress-text";
			}
		}
		
		//==================================================
		//	Disable this progressbar
		//==================================================
		disable(bParent)
		{
			super.disable(bParent);
			
			this.complete.className = "progress-on-disabled";
			if(this.label)
				this.label.className = "progress-text-disabled";
		}
		
		//==================================================
		//	Set progress value
		//==================================================
		setValue(fValue)
		{
			if(typeof fValue != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_VALUE, this.id),
											"progressbar.js", "jaswin.progressbar.ProgressBar.setValue");
			
			if(!this.enabled)
				return;
			
			this.value = fValue;
			if(this.value > this.total)
				this.value = this.total;
			
			let iRatio = (this.value / this.total * 100).toFixed(0);
			this.complete.style.width = iRatio + "%";
			
			if(this.style != progressbar.STYLE_DEFAULT)
			{
				this.label.innerHTML = iRatio + "%";
				
				if(this.style == progressbar.STYLE_DETAIL)
				{
					this.label.innerHTML += " (" + this.value.toFixed(this.digit)
											+ "/" + this.total.toFixed(this.digit) + this.unit + ")";
				}
			}
		}
		
		//==================================================
		//	Set total value
		//==================================================
		setTotal(fTotal)
		{
			if(typeof fTotal != "number")
				throw new jaswin.JaswinError(jaswin.ERR_ALERT,
											jaswin.formatMessage(jaswin.ALERT_TYPE_TOTAL, this.id),
											"progressbar.js", "jaswin.progressbar.ProgressBar.setTotal");
			
			if(!this.enabled)
				return;
			
			this.total = fTotal;
			if(this.value > this.total)
				this.value = this.total;
			
			let iRatio = this.getRatio();
			this.complete.style.width = iRatio + "%";
			
			if(this.style != progressbar.STYLE_DEFAULT)
			{
				this.label.innerHTML = iRatio + "%";
				
				if(this.style == progressbar.STYLE_DETAIL)
				{
					this.label.innerHTML += " (" + this.value.toFixed(this.digit)
											+ "/" + this.total.toFixed(this.digit) + this.unit + ")";
				}
			}
		}
		
		//==================================================
		//	Get ratio
		//==================================================
		getRatio()
		{
			let iRatio = 0;
			if(this.total)
				iRatio = (this.value / this.total * 100).toFixed(0);
			
			return iRatio;
		}
	}
}(jaswin.progressbar = jaswin.progressbar || {}));
